/**  
 * @doc Raw_Proto.hpp 
 *  This file contains class interface of Raw output Protocol
 */

#ifndef __RAW_PROTO_HPP__
#define __RAW_PROTO_HPP__

#include <vector>
#include <string>
using namespace std;

#include "VLBI_Proto.hpp"
#include "ExtSubCh.hpp"

extern "C"{
#include "stdint.h"
}

/** 
 *  Raw_Protocol simply outputs data without any header
 *  Succeeding from VLBI_Proto class
 */

class RAW_Proto:public VLBI_Proto {
  RAW_Proto(RAW_Proto& rp){;} ///< Copy Constructor, prohibited.
public:
  /** Constructor  */
  RAW_Proto():VLBI_Proto(TransInfo::RAW_Proto, TransInfo::FILE_IO){;}
  /// Destructor
  ~RAW_Proto(){;}
  /** Transmitting single data stream with protocol */
  int32_t TX1sec(const time_t t, vector <uint8_t>& arr);
  /** Transmitting multiple  data stream with protocol */
  int32_t TX1secSub(const time_t t, vector <uint8_t>& arr, const DAS_Info* das=0);
  /**  Transmitting data from Buffer for 1sec, pointer version */
  int32_t TX1sec(const time_t t,  const uint8_t* arr, const int32_t ndat);
  /// Transmitting subset channels of 1sec of data, pointer version 
  int32_t TX1secSub(const time_t t,  const uint8_t* arr, const int32_t ndat, const DAS_Info* das=0);
  //  virtual int32_t TX1sec(time_t t, uint8_t* arr, const DAS_Info* dasinf)=0;
  int32_t RX_UDP(void* buf, const int32_t buf_size);
  int RXvdif(DataPage& dpage);
  int ApplProtoHeadSize()const{return 0;}
};

#endif
