/** CMPLX.HPP
 *  Doouble/Float Complex Class and related functions.
 * @auhtor M.Sekido
 **/

#ifndef __Cmplx__
#define __Cmplx__

#include <vector>
#include <cstdio>
#include <cmath>
#include <functional>
#include <algorithm>
//using namespace std;

//#include "const2.hpp"
/** DComplex 
 * @doc Complex Class with double precision
 *
 **/
const double 	PI =	M_PI ;
const double 	TWOPI =	2.0 * M_PI ;


class DComplex {
  double re;
  double im;
  //  double amp;
  //  double phs;
 public:
  DComplex(double r=0.0, double i=0.0){ re = r; im = i;}
  DComplex(void){re=0.0; im=0.0;}
  friend DComplex operator+(const DComplex& x, const DComplex& y){
    return(DComplex(x.re + y.re, x.im + y.im));
  }
  friend DComplex operator+(const double& x, const DComplex& y){
    return(DComplex(x + y.re, y.im));
  }
  friend DComplex operator+(const DComplex& x, const double& y){
    return(DComplex(x.re + y, x.im));
  }
  DComplex operator+(void) {return(*this);}
  DComplex operatori(void) {return(DComplex(-re, -im));}
  DComplex operator+=(DComplex x) {
    re += x.re;
    im += x.im;
    return(*this);
  }
  DComplex operator-=(DComplex x) {
    re -= x.re;
    im -= x.im;
    return(*this);
  }
  double Real(void){return(re);}
  double Imag(void){return(im);}
};

/** FDComplex 
 * @doc Complex Class with single precision
 *
 **/

class FComplex {
  float re;
  float im;
 public:
  FComplex(float r, float i=0.0){ re = r; im = i;}
  FComplex(void){ re=0.0; im=0.0;}
  /** Copy Constructur **/
  FComplex(const FComplex & c){ re=c.Real(); im=c.Imag();}
  friend FComplex operator+(const FComplex& x, const FComplex& y){
    return(FComplex(x.re + y.re, x.im + y.im));
  }
  friend FComplex operator+(const float& x, const FComplex& y){
    return(FComplex(x + y.re, y.im));
  }
  friend FComplex operator*(const FComplex& x, const FComplex& y){
    return(FComplex(x.re*y.re - x.im*y.im, x.re*y.im + x.im*y.re));
  }
  friend FComplex operator/(const FComplex& x, const FComplex& y){
    double amp = (y.re*y.re + y.im*y.im);
    FComplex ctmp;
    ctmp.re = (x.re*y.re + x.im * y.im)/amp;
    ctmp.im = (y.re*x.im - x.re * y.im)/amp;
    return(ctmp);
  }
  friend FComplex operator+(const FComplex& x, const float& y){
    return(FComplex(x.re + y, x.im));
  }
  friend FComplex operator*(const FComplex& x, const float& y){
    return(FComplex(x.re * y, x.im *y));
  }
  FComplex& operator=(const float x) {re=x; return(*this);}
  FComplex& operator()(const float x, const float y) {re=x; im=y; return(*this);}
  FComplex& operator+(void) {return(*this);}
  FComplex operator-(void) {return(FComplex(-re, -im));}
  FComplex& operator+=(FComplex x) {
    re += x.re;
    im += x.im;
    return(*this);
  }
  FComplex& operator-=(FComplex x) {
    re -= x.re;
    im -= x.im;
    return(*this);
  }
  FComplex operator*=(FComplex x) {
    FComplex c1 = *this;
    re = c1.re*x.re - c1.im*x.im;
    im = c1.re*x.im + c1.im*x.re;
    return(*this);
  }
  FComplex operator/=(FComplex x) {
    FComplex c1 = *this;
    double amp = x.re*x.re + x.im+x.im;
    re = (c1.re*x.re + c1.im*x.im)/amp;
    im = -(c1.re*x.im - c1.im*x.re)/amp;
    return(*this);
  }
  FComplex operator/=(float x) {
    re /= x;    im /= x;
    return(*this);
  }
  FComplex operator*=(float x) {
    re *= x;
    im *= x;
    return(*this);
  }
  bool operator<( const FComplex & x) {
    double x1 = sqrt((double)re*re+(double)im*im);
    float  x2 = x.Amp();
    return( x1 < x2);
  }
  bool operator>(const FComplex & x) {
    float x1 = (float)sqrt((double)re*re+(double)im*im);
    float x2 = x.Amp();
    return( x1 > x2);
  }
  bool operator==(const FComplex & x) {
    float x1 = (float)sqrt((double)re*re+(double)im*im);
    float x2 = x.Amp();
    return( x1 == x2);
  }
  
  void Print_c(FILE *fp=stdout){ 
    fprintf(fp, "(%10.3e, %10.3e)\n", re, im);
  }
  void  xk(float k){re*=k; im*=k;}
  float Real(void) const {return(re);}
  float Imag(void) const {return(im);}
  float Amp(void) const { return(sqrt((double)re*re+(double)im*im));}
  float Phase(void) const {return(atan2(im,re));}
  void Set_val(float r, float i){ re = r; im = i;}
  FComplex& Conj(void){  im *= -1.;  return (*this);}
};

class Cmaxmin_db{
  const FComplex *cfarr;
  const DComplex *cdarr;
  FComplex cmax, cmin;
  double snr,a_mean;
  long imax, imin, jmax, jmin;
  long ndat[2];
 public:
  Cmaxmin_db(void){imax=imin=jmin=jmax=0; }
  void Max_min1(const FComplex*, long);
  void Max_min2(const FComplex*, long, long);
  double Max(void)const {return cmax.Amp();}
  double Min(void)const {return cmin.Amp();}
  double Max_phs(void)const{return cmax.Phase();}
  double Min_phs(void)const {return cmin.Phase();}
  long Imax(void)const {return(imax);}
  long Imin(void)const {return(imin);}
  long Jmax(void)const {return(jmax);}
  long Jmin(void)const {return(jmin);}
  double Snr(void)const {return(snr);}
  double Mean(void)const {return a_mean;}
};

/**  FCarr2Farr() 
 * @doc Convertion from FComplex-array to Real/Imag arrays.
 *
 **/
inline void FCarr2Farr(FComplex* ca, float *yarr1, float *yarr2, int n){
  int i;
  for(i=0; i<n; i++){
    yarr1[i] = ca[i].Real();
    yarr2[i] = ca[i].Imag();
  }
	 
}

/**  FCarr2Farr() 
 * @doc Convertion from FComplex-array to Amp. and Phase  arrays.
 *
 **/
inline void FCarr2AParr(std::vector <FComplex>& ca, float *amp_arr,  float *phs_arr){
  
  long n = ca.size();
  for(long i=0; i<n; i++){
    amp_arr[i] = ca[i].Amp();
    phs_arr[i] = ca[i].Phase(); // Radian
  }
}

inline void FCarr2AParr(std::vector <FComplex>& ca, std::vector <float> &amp_arr,  
			std::vector <float> &phs_arr){
  
  long n = ca.size();
  if( amp_arr.size() < (size_t)n) amp_arr.resize(n);
  if( phs_arr.size() < (size_t)n) phs_arr.resize(n);
  for(long i=0; i<n; i++){
    amp_arr[i] = ca[i].Amp();
    phs_arr[i] = ca[i].Phase();  // Radian
  }
}

inline void FCarr2AParr(FComplex* ca, float *amp_arr, float *phs_arr, int n){
  int i;
  for(i=0; i<n; i++){
    amp_arr[i] = ca[i].Amp();
    phs_arr[i] = ca[i].Phase();  // Radian
  }
}

/****
 A[n x m] ->B[m x n]
**/

inline void Trn_C(FComplex* a, FComplex* b, int n, int m){
  int i,j;
  for(i=0; i<n; i++){
    for(j=0; j<m; j++){
      b[i*m+j] = a[j*n+i];
    }
  }
}

inline void Trn_C(std::vector <FComplex>& a, std::vector <FComplex>& b, int n, int m){
  for(int i=0; i<n; i++){
    for(int j=0; j<m; j++){
      b[i*m+j] = a[j*n+i];
    }
  }
};
/*
void FCarr2Farr(FComplex* ca, float *yarr1, float *yarr2, int n);
void FCarr2AParr(std::vector <FComplex>& ca, float *amp_arr, float *phs_arr);
void FCarr2AParr(FComplex* ca, float *amp_arr, float *phs_arr, int n);
void Trn_C(FComplex* a, FComplex* b, int n, int m);
*/

template <class T> 
struct Eular:public std::unary_function <FComplex,T>{
  FComplex operator()(const T phs)const{
    double d = fmod((double)phs,TWOPI);
    return FComplex (cos(d),sin(d));
    //    fprintf(stderr, "phs=%.2lf, (%.2f, %.2f) \n",  d, c.Real(), c.Imag());
    //  return c;
  }
};

template <typename T>
struct fc_mplx:public std::binary_function <FComplex,FComplex,T>{
  FComplex operator()(const FComplex x, const T f)const{
    
    return x * f; }
};


#endif

