
#ifndef __K5VSI_MDL_HPP__
#define __K5VSI_MDL_HPP__

#include <string>
#include <vector>
#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <cmath>
#include <iostream>
using namespace std;

#include "UDPSocket.h"
#include "TCPSocket.h"
#include "FileSocket.h"
#include "Thread.h"
#include "netutl.h"

#include "InDev_mdl.hpp"
#include "bitutl.hpp"

// #include "m5b.hpp"

extern "C" {
#include <stdint.h>
#include <limits.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/types.h>  
#include <sys/socket.h>  
#include <sys/time.h>  
#include <string.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/mman.h>
#include <pthread.h>
#include <signal.h>
#include "k5vsi.h"
}

// const int MAXSTR =           200; 
//const int DEFAULT_MTUSIZE = 1500;
//const int DEFAULT_NPACKET = 100000;
//const int DEFAULT_SNDR_PORT =    52100;
const int DEFAULT_LENGTH =      10;
// const int SNDBUFSIZE    =  16777216;
// mark5 default ports
//const int DEF_M5CTL_PORT =     2620;
//const int DEF_M5DAT_PORT =     2630;


// int setup_net(char *hostname, unsigned short port, int *sock);
// void my_usleep(u_int64_t usec);

//void mark5header(unsigned int header[4], time_t epoch, double sec);


int make_tab(unsigned char table[256]);
int make_tab_ni(unsigned char table[256]);
int make_tab_sgnmag(unsigned char table[256]);

/** 
 * Input model for Data Reding from PC-VSI (K5/VSI) Device. 
 * Succeeding from InDev_Model.
 */

class K5_VSI:public InDev_Model{
public:
  K5_VSI(); ///<  Constructor 
  ~K5_VSI(); ///< Destructor 
  const int OpenDev(); ///< K5-VSI Device Open 
  const int CloseDev();///< K5-VSI Device Close 
  const int TestDev(); ///< Test device/file Permission  
  /** This is dummy in k5vsi_mdl class. */
  //  const IndevReturns_t MappingData(vector <uint8_t>& arr,time_t epoch);
  /** Getting page address of the epoch, rpage is returning page number. */
  const uint8_t* MappingData(time_t epoch); 
  /// Release memry page address of the epoch, rpage is returning page number.
  const IndevReturns_t  UnMapping(const time_t epoch=0);
  /**  Return page_size(1sec) in 4-byte unit  */
  uint32_t PageLength(){return PageLength(0);}  
  uint32_t PageLength(const int ipage); ///< Returns Pagesize in word
  /// Return device type in string 
  string StrDevType(){string ret("K5/VSI"); return ret;}
  /// Returns Device type 
  InputModelType_t DevType(){return InDev_Model::K5VSI;}
  int InitPage(){return 0;} ///<  Do nothing but for compatibility
  //  const int Set_srate(uint32_t a); ///< Set Sampring rate (sps)
  const int32_t Srate()const; ///< Return Sampling rate (sps)
  //  const int Set_Nch(const int a); ///< Set Nch
  const int Nch()const; ///< Return Nch; 
  //  int Set_Nbit(const int a); ///< Set Quantization bits
  const int Nbit()const; ///< Return Quantization bits
  //  void Set_Length(const char* a); ///< Set Recording duration
  //  void Set_Length(const int a){length=a;} ///< Set Recording duration
  //  const uint32_t Length()const {return length;} ///< Return Recording Duration
  const int Set_Mode(const char* a); ///< Set Observation mode
  const int ISet_Mode();  ///< Setting Observation mode from (srate,nch)
  string Mode(); ///< Return Obseravtion mode
  void Set_iGrp(const int i){igrp=i;} ///< Set group No. from 32/nch groups
  const int IGrp(void)const{ return igrp;} ///< Return Selected group Number.
  /// Returns top address of that page
  uint32_t Page_Offset(const int ipage){  
    return (uint32_t)ioctl(device,GET_FRAME_OFFSET,ipage);}
  /// Get Device Epoch from PC-VSI board
  time_t   GetDevEpoch();
  /// Get Epoch of the ipage;
  time_t   GetFrameEpoch(const int ipage)const;
  /// Returning epcoh of the ipg page
  time_t   Epoch(const int ipg)const {return GetFrameEpoch(ipg);}
  /// Converting raw data to requested data rate/bit/nch
  int32_t Convert(vector <uint8_t> & arr, int offset);
  int FirstPage(); ///< return page number of earliest epoch
  /// Return Page state in string 
  string StrPageState(const int lvl=0){ string ret="NORMAL"; return ret;}
  /// Fill DATA_END to all pages
  void PageEnd(){map_enable=false;} // do nothing
protected:
  int          device; ///< file descriptor of VSI device
  int             mem; ///< file descriptor of memory device
  time_t device_epoch; ///< Epoch obtained from K5-VSI
  string vsi_devname; ///< K5-VSI Device file name
  string mem_devname; ///< K5-VSI Memory Device file name
  int npage;  ///< Number of available pages;
  //  int nch;   ///< Nch
  //  int nbit;  ///< Quantization bits
  int igrp;  ///< Group of 32/nch; 
  //  uint32_t speed; ///< Output rate (Word/sec)
  //  uint32_t srate;  ///< Smapling rate(sps)
  //  uint32_t length; ///< sec for recording
  //  string mode;  ///< Srate/bit/nch information in string 
  uint8_t enc_table[256]; ///< Encoding Table 
  uint8_t *page_ptr;  ///< Page pointer for current reading 
  int rpage;  ///< mapped page number
  bool map_enable; /// Mapping Enable state, used for stop running.
};

inline  time_t   K5_VSI::GetDevEpoch(){
  if( device ==0){
    cerr << "Error in GetDevEpoch(): device is not opened yet\n";
    return -1;
  }
  return ioctl(device,GET_DEVICE_EPOCH,NULL);
}

inline  time_t   K5_VSI::GetFrameEpoch(const int ipage)const{
  if(ipage>= npage){
    fprintf(stderr,
	    "Error in K5_VSi::FrameEpoch(): requesting page(%d) > npage(%d)\n",
	    ipage,npage);
    return 0;
  }
  return  ioctl(device,GET_FRAME_EPOCH,ipage);
}

inline  uint32_t  K5_VSI::PageLength(const int ipage){
  if(ipage>= npage){
    fprintf(stderr,"Error in K5_VSi::PageLength(): requesting page(%d) > npage(%d)\n",
	    ipage,npage);
    return 0;
  }
  uint32_t page_len= (uint32_t)ioctl(device,GET_FRAME_CLOCK ,ipage);
  return page_len;
}

/******** K5_VSI_SENDER from here  ************* ****/
// enum Prot_t {PROTO_TCP,PROTO_UDP}; 
// const uint32_t WORK_BUF_SIZE = 65536;

// enum Encoding_mode {ENC_SGNMAG, ENC_OFSTBIN, ENC_NOCHANG};

// class K5_VSI_SENDER:public K5_VSI{
// public:
//   enum State_t {STATE_IDLE, STATE_RUN, STATE_QUIT}; 
// private:
//   State_t state;
//   unsigned int data_port;
//   //  unsigned int packetsize;
//   int npacket; 
//   string hostname;
//   /** @doc socket address structure */
//   struct sockaddr_in addr;
//   vector <char> dummy_buf;
//   Socket* sock_;
//   UDPSocket* udpsock_;
//   int mtu;
//   int sockBufsize; 
//   Prot_t data_protocol;
//   //  Mk5B_Frame m5bf;
//   // -- for mk5data packet creation
//   //  unsigned long long *sequence; 
//   uint64_t *sequence; 
//   uint8_t  table[256];
//   Encoding_mode encoding;
//   vector <uint8_t>  dat;
//   vector <uint8_t>  packet;
//   string outfile;
//   FILE *ofp;
//   //  FileSocket* fout_sock_;
// public:
//   K5_VSI_SENDER():state(STATE_IDLE),data_port(DEF_M5DAT_PORT),
// 		  npacket(DEFAULT_NPACKET),hostname("vlbi2"),
// 		  sock_(NULL), udpsock_(NULL), mtu(DEFAULT_MTUSIZE),
// 		  sockBufsize(-1),
// 		  data_protocol(PROTO_UDP),sequence(NULL),
// 		  encoding(ENC_OFSTBIN),
// 		  dat(WORK_BUF_SIZE),packet(1500),ofp(NULL){;}
//   void Set_Port(const uint32_t a){data_port=a;}
//   void Set_Npacket(const uint32_t a){npacket=a;}
//   void Set_Host(const char* a){hostname = a;}
//   string & Host(){return hostname;}
//   void Init_Buf();
//   const int Setup_Net();
//   //  const int Setup_Net2();
//   void Close_Net();
//   void Set_Protocol(Prot_t pt){data_protocol=pt;}
//   void Set_mtu(const int m){mtu=m;}
//   const int Get_mtu()const {return mtu;}
//   void Sock_Create();
//   ~K5_VSI_SENDER();
//   //  const int Send_Mk5();
//   //  const int Send_Mk5b();
//   void Set_Idle(){state=STATE_IDLE;}
//   const int State()const{ if(state == STATE_RUN) return 1; else return 0;}
//   void Set_outfile(const char* a);
//   //  Mk5B_Frame* const pM5B() {return &m5bf;}
//   void Set_Encoding(Encoding_mode e){encoding=e;}
//   const Encoding_mode Encoding()const {return encoding;}
// };

// inline K5_VSI_SENDER::~K5_VSI_SENDER(){
//   if(sock_ != NULL){    delete sock_; sock_=NULL; udpsock_=NULL; }
//   //  if(fout_sock_ != NULL){    delete fout_sock_; fout_sock_=NULL;  }
//   if(ofp!=NULL){ fclose(ofp); ofp=NULL;}
// }


// class K5VSI_Srv:public Thread{
//   Prot_t protocol;
//   K5_VSI_SENDER *k5vsi_sender;
//   Socket *srv_sock_;
// public:
//   K5VSI_Srv():protocol(PROTO_UDP),
// 	      k5vsi_sender(NULL),
// 	      srv_sock_(NULL){;}
//   void* thread_func();
//   ~K5VSI_Srv();
//   void Set_Sender(K5_VSI_SENDER *a){ k5vsi_sender=a;}
//   void Set_SrvSock(Socket* s){srv_sock_=s;}
//   const int CommunicateWithClient();
//   const int CmdParser(const char *buf, Socket* s);
//   const int Send_start();
// };

// inline K5VSI_Srv::~K5VSI_Srv(){
//   k5vsi_sender=NULL;  
// }


#endif

