/**  Time_lib is a class for manupulate time data 
 * @author M.Sekido
 */

#ifndef __TIME_LIB__
#define __TIME_LIB__

#include <string>
#include <vector>
#include <iostream>
//using namespace std;

#include  "utlty.hpp"
// #include  "strngutl.hpp"
#include <boost/algorithm/string.hpp>

#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <ctime>

extern "C" {
#include <stdint.h>
#include <sys/time.h>
};

int Str2Tint(char *a);

inline double GetTime(){
  struct timeval et;
  gettimeofday(&et,NULL);
  double tdif = et.tv_sec+0.000001*et.tv_usec;
  return tdif;
}

/** time for Half-Year counter and total second from the epoch.
 * This time coding is used in VDIF (VLBI Data Interexchange Format) format.
 */
class HY_Time{
  /** half year counter from 2000/1/1 */
  uint8_t hy_count;
  /** Total second from the half year counter hy_count */
  double tsec; 
public:
  /** Default Constructor  */
  HY_Time(const int hy=0, const double s=0):hy_count(hy),tsec(s){}
  /** Copy Constructor */
  HY_Time(const HY_Time &hy){*this = hy;}
  /** Set Half-year conter */
  void Set_hy(const int hy){ hy_count = hy;}
  /** Set Total second form the Half-year epoch */
  void Set_tsec(const double sec){ tsec = sec;}
  /** Returning Total second form the Half-year epoch */
  double Tsec()const {return tsec;}
  /** Returning half year conter from 2000.0 */
  int8_t Hyear_cnt()const {return hy_count;}
};


/** Class of time */
class Time_dat{
  /** MJD  */
  double mjd;
  /** Two experessions of epoch **/
  int ymdhms[6], ydhms[5];
  double sec;
  double soy, doy, sod;
 public:
  Time_dat(void){ 
    int i;
    for(i=0; i<6; i++) ymdhms[i] =0;
    for(i=0; i<5; i++) ydhms[i] =0;
    mjd = doy = sec =soy =sod =0.0;
  }
  Time_dat(const int yy, const int doy, const int hh, 
	   const int mm, const double ss);
  Time_dat(const int yy, const int mon, const int dd, const int hh, const int mm, const double ss);
  Time_dat(const double mjd_in);
  /** Copy Constructor  */
  Time_dat(const Time_dat &t0){Set_time(t0);}
  Time_dat(const time_t t1){Set_time(t1);}
  ~Time_dat(){
    //    std::cerr << " Time_dat destructor \n";
  }
  /** Set up time info*/
  int Set_time(const Time_dat &t0);
  int Set_time(const int yy, const int doy, const int hh, 
		     const int mm, const double ss);
  int Set_time(const int yy, const int mon, const int dd, const int hh, const int mm, const double ss);
  int Set_time(const double mjd_in);
  int Set_time(const int yy, const double soy);
  int Set_time(const char* str , const int nterm, const char* del);
  int Set_time(const std::string &str , const int nterm, const char* del);
  int Set_time(const char str[]); /* yyyydddhhmmss */
  int Set_time(const time_t t1);
  void Set_time_to_now(void);
  void Doy2md(void);
  double Mjd_cal(void);
  double Doy_cal(void);
  void Sec2dhms(double);
  void Sec2dhms(const int yy, const double s){
    ydhms[0] = ymdhms[0] = yy; Sec2dhms(s); Doy2md(); Mjd_cal();}
  /** Print time with a format 
   * - iptn=0  mjd
   * - iptn=10 yyyy/ddd hh/mm/ss.mmm
   * - iptn=11 YYYYyDDDDdHHhMMmSS.SSS
   * - iptn=12 YYYY DDDD HH MM SS
   * - iptn=107 YYYY/DDDD HH:MM:SS.77777
   * - iptn=20 yyyy/mm/dd hh/mm/ss.mmm
   * - iptn=21 yyyy/mm/dd hh/mm/ss
   * - iptn=32 hh mm ss.mmmmmmm
  */
  void Print_time(FILE *fp,int iptn=0  ) const;
  void Print_time(std::string &str, int iptn=0  ) const;
  std::string Print_time(const int iptn=0  ) const;
  //  void Str_time(std::string &buf, int iptn=0 );
  double Mjd(void)const {return(mjd);}
  double JD(void) const {return(Mjd() + 2400000.5);}
  double Doy(void)const {return(doy);}
  double Soy(void)const {return(soy);}
  double Sod(void)const {return(sod);}
  double Sec(void)const {return sec;}
  int Year(void)const {return(ymdhms[0]);}
  int Month(void)const {return(ymdhms[1]);}
  int Day(void)const {return(ymdhms[2]);}
  Time_dat Time_0h(void){return(Time_dat(ydhms[0], ydhms[1],0,0,0));}
  bool operator==(const Time_dat &y) const;
  bool operator!=(const Time_dat &y) const;
  bool operator<(const Time_dat &y)const {return(Mjd() < y.Mjd());}
  bool operator>(const Time_dat &y)const {return( Mjd() > y.Mjd());}
  Time_dat operator+(const Time_dat& y) const;  
  Time_dat& operator+=(const double sec);
  Time_dat& operator-=(const double sec);
  friend Time_dat operator+(const Time_dat& x, double sec) ;
  friend Time_dat operator-(const Time_dat& x, double sec) {
    return((x + (-sec))); }
  double operator-(const Time_dat& y) const;
  inline void err_msg(const int imod, const double val);
  double TAI_UTC(void);
  bool Check_timedat(void);
  HY_Time Hy_time() const;
  time_t Time_t()const;
};

inline  int Time_dat::Set_time(const Time_dat& t0){
  if(this == &t0) return 0;
  mjd = t0.mjd;
  sec = t0.sec;
  soy = t0.soy;
  doy = t0.doy;
  sod = t0.sod;
  memcpy(ymdhms,t0.ymdhms, 6*sizeof(int));
  memcpy(ydhms,t0.ydhms, 5*sizeof(int));
  return 0;
}

inline  Time_dat& Time_dat::operator-=(const double sec){
    *this = *this - sec;
    return(*this);
};

inline  Time_dat& Time_dat::operator+=(const double sec){
    *this = *this + sec;
    return(*this);
};

inline bool Time_dat::operator!=(const Time_dat &y) const{
  return(ydhms[0] != y.ydhms[0] ||
	 ydhms[1] != y.ydhms[1] ||
	 ydhms[2] != y.ydhms[2] ||
	 ydhms[3] != y.ydhms[3] ||
	 ydhms[4] != y.ydhms[4] );  
}

inline  bool Time_dat::operator==(const Time_dat &y) const{
  return(ydhms[0] == y.ydhms[0] &&
	 ydhms[1] == y.ydhms[1] &&
	 ydhms[2] == y.ydhms[2] &&
	 ydhms[3] == y.ydhms[3] &&
	 ydhms[4] == y.ydhms[4] );  
}

inline double Time_dat::operator-(const Time_dat& y) const{
  
  double imjd = floor(Mjd());
  double jmjd = floor(y.Mjd());
  double isec = Sod();
  double jsec = y.Sod();
  double v = (imjd - jmjd )* 86400.+ isec - jsec;
  
  return(v);
}


inline void Time_dat::err_msg(const int imod, const double val){
  switch (imod){
  case 0:  
    std::cerr << "Error doy(" << val << ") out of range";
    break;
  case 1:
    std::cerr << "Error mon(" << val << ") out of range";
    break;
  case 2:
    std::cerr << "Error dd(" << val << ") out of range";
    break;
  case 3:
    std::cerr << "Error hh(" << val << ") out of range";
    break;
  case 4:
    std::cerr << "Error mm(" << val << ") out of range";
    break;
  case 5:
    std::cerr << "Error ss(" << val << ") out of range";
    break;
  default:
    break;    
  }
  return;
}

inline int  Time_dat::Set_time(const int yy, const double soy){ 
  //  Time_dat t_tmp(yy,1,1,0,0,0);
  //  *this = t_tmp + soy;
  this->Set_time(yy,1,1,0,0,0);
  *this += soy;
  return 0;
}

int index_tarr(std::vector <Time_dat>& tarr, Time_dat t_ref);
int Smonth(const std::string mon);

inline void Time_dat::Set_time_to_now(void){
  struct tm *now;
  time_t  aclock;
  time(&aclock);
  now = gmtime(&aclock);
  this->Set_time(now->tm_year + 1900, now->tm_yday+1,now->tm_hour,
		 now->tm_min,(double)now->tm_sec);
}

/** Compute t1 -  t2 and returns difference in usec */
inline int32_t TimeDiffUsec(const struct timeval& t1, const struct timeval& t2){
  time_t dt1 = t1.tv_sec - t2.tv_sec ;
  time_t dt2 = t1.tv_usec - t2.tv_usec ;
  int32_t dt = dt1 * 1000000 + dt2;
  return dt;
} 

#endif
